package com.lodgon.dali.core.social.resources;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.ejb.UserBean;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.Constants;
import com.lodgon.dali.core.social.ExternalNetwork;
import com.lodgon.dali.core.social.ejb.DaliCoreSocialBean;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.oauth.signature.OAuthParameters;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import javax.annotation.ManagedBean;
import javax.inject.Inject;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.*;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * 
 * This class handles the interaction between clients (web client, JavaFX client)
 * and the authentication functionality of social networks.
 * <br/>
 * Connecting to a social network is done by sending the user to the URL
 * <code>/connect/{network}</code> which calls the connect-method.
 * The user will be redirected to the network-specific authentication 
 * mechanism. If a <code>callback</code> parameter is specified, the user will
 * be redirected to the URL defined by the <code>callback</code> parameter.
 * <p>
 * We don't expose accounts, streams and friends as part of the DaliCoreSocial
 * rest handler, since these should be subject to permission checks in specific
 * applications. The corresponding EJB methods can be called by
 * application-specific REST handlers.
 *
 *
 * @author joeri
 */
@ManagedBean
@Path("/dalicoresocial")
public class DaliCoreSocialHandler {

	@Inject
	DaliCoreSocialBean dcsBean;
	@Inject
	UserBean userBean;
	@Context
	ServletContext servletContext;
	@Context
	HttpServletRequest request;
	@Context
	UriInfo ui;
						

	/**
	 * Connect to an external network.
	 *
	 * @param network
	 * @param callback
	 * @return
	 */
	@GET
	@Path("/connect/{network}")
	public Response connect(@PathParam("network") String network,
					@QueryParam("daliKey") @DefaultValue("") String appKey,
					@QueryParam("callback") String callback) {
		Constants.LOGGER.log(Level.FINE, "ExternalNetworkhandler: Connecting anonymous user to network {0}, with callback {1} for appkey {2}", new Object[]{network, callback, appKey});
		request.getSession().removeAttribute(ExternalNetwork.DALICORESOCIAL_USER);
		request.getSession().removeAttribute(ExternalNetwork.DALICORESOCIAL_UPDATEUSER);
		request.getSession().setAttribute(ExternalNetwork.daliKey, appKey);

		Map<String, String> initParameters = new HashMap<String, String>();
		for (Enumeration<String> initParameterNames = servletContext.getInitParameterNames(); initParameterNames.hasMoreElements();) {
			String initParameterName = initParameterNames.nextElement();
			initParameters.put(initParameterName, servletContext.getInitParameter(initParameterName));
		}

		ExternalNetwork externalNetwork = dcsBean.getExternalNetwork(appKey, network, initParameters);
		if (externalNetwork != null) {
			return externalNetwork.connect(request, ui, callback);
		} else {
			return Response.ok("The specified network or application are invalid or configuration was missing from the application settings.").status(Response.Status.BAD_REQUEST).build();
		}
	}

	/**
	 * Connect a known user to an external network
	 *
	 * @param network
	 * @param uid the UUID of the user
	 * @param callback
	 * @return
	 * @throws DaliCoreException
	 */
	@GET
	@Path("/connect/{network}/{uid}")
	public Response connect(@PathParam("network") String network,
					@PathParam("uid") String uid,
					@QueryParam("daliKey") @DefaultValue("") String appKey,
					@QueryParam("callback") String callback,
					@DefaultValue("false") @QueryParam("update") Boolean update) throws DaliCoreException {
		User user = userBean.getByUid(appKey, uid);
		Constants.LOGGER.log(Level.FINE, "external Connecting user {0} to network {1}, with callback {2} for appkey {3}", new Object[]{user, network, callback, appKey});
		request.getSession().setAttribute(ExternalNetwork.DALICORESOCIAL_USER, user);
		request.getSession().setAttribute(ExternalNetwork.DALICORESOCIAL_UPDATEUSER, update);
		request.getSession().setAttribute(ExternalNetwork.daliKey, appKey);

		Map<String, String> initParameters = new HashMap<String, String>();
		for (Enumeration<String> initParameterNames = servletContext.getInitParameterNames(); initParameterNames.hasMoreElements();) {
			String initParameterName = initParameterNames.nextElement();
			initParameters.put(initParameterName, servletContext.getInitParameter(initParameterName));
		}

		ExternalNetwork externalNetwork = dcsBean.getExternalNetwork(appKey, network, initParameters);
		if (externalNetwork != null) {
			return externalNetwork.connect(request, ui, callback);
		} else {
			return Response.ok("The specified network or application are invalid or configuration was missing from the application settings.").status(Response.Status.BAD_REQUEST).build();
		}
	}

	/**
	 * Remove a specific network from the list of connected networks for a given
	 * user // TODO: add authentication to this call.
	 *
	 * @param network
	 * @param uid
	 * @return
	 */
	@GET
	@Path("/disconnect/{network}/{uid}")
	public Response disconnect(@PathParam("network") String network,
					@PathParam("uid") String uid,
					@QueryParam("daliKey") @DefaultValue("") String appKey) throws DaliCoreException {
		dcsBean.disconnect(appKey, network, uid);
		return Response.ok().build();
	}

	@GET
	@Produces(MediaType.TEXT_PLAIN)
	@Path("/callback/{network}")
	public Response callback(@PathParam("network") String network) throws URISyntaxException {
		String appKey = (String) request.getSession().getAttribute(ExternalNetwork.daliKey);
		User user = (User) request.getSession().getAttribute(ExternalNetwork.DALICORESOCIAL_USER);
		Boolean updateUser = (Boolean) request.getSession().getAttribute(ExternalNetwork.DALICORESOCIAL_UPDATEUSER);

		Map<String, String> initParameters = new HashMap<String, String>();
		for (Enumeration<String> initParameterNames = servletContext.getInitParameterNames(); initParameterNames.hasMoreElements();) {
			String initParameterName = initParameterNames.nextElement();
			initParameters.put(initParameterName, servletContext.getInitParameter(initParameterName));
		}

		ExternalNetwork externalNetwork = dcsBean.getExternalNetwork(appKey, network, initParameters);
		if (externalNetwork != null) {
			MultivaluedMap<String, String> params = ui.getQueryParameters();
			String oauthRequestToken = params.getFirst(OAuthParameters.TOKEN);
			String oauthVerifier = params.getFirst(OAuthParameters.VERIFIER);
			if (params.containsKey("code")) {
				oauthRequestToken = params.getFirst("code");
				oauthVerifier = ui.getBaseUri().getScheme() + "://" + ui.getBaseUri().getAuthority() + ui.getBaseUri().getPath() + "dalicoresocial/callback/" + network;
			}

			ExternalToken accessToken = externalNetwork.callback(oauthRequestToken, oauthVerifier);
			try {
				OnlineAccount onlineAccount = externalNetwork.registerOnlineAccount(appKey, user, accessToken, updateUser);
				String verifier = dcsBean.createVerifier(onlineAccount);
				
				String localCallback = (String) request.getSession().getAttribute(ExternalNetwork.DALICORESOCIAL_LOCALCALLBACK);
				if (localCallback == null || localCallback.trim().isEmpty() || "oob".equals(localCallback.trim())) {
					return Response.ok(onlineAccount.getUser().getUid()).build();
				} else {
					if (localCallback.indexOf("?") == -1) {
						localCallback += "?";
					} else {
						localCallback += "&";
					}
					localCallback += "uid=" + onlineAccount.getUser().getUid();
					localCallback += "&verifier=" + verifier;

					return Response.seeOther(new URI(localCallback)).build();
				}
			} catch (DaliCoreException ex) {
				StringWriter sw = new StringWriter();
				PrintWriter pw = new PrintWriter(sw);
				ex.printStackTrace(pw);
				pw.close();
				throw new WebApplicationException(ex, Response.ok(sw.getBuffer().toString()).status(Response.Status.BAD_REQUEST).build());
			}
		} else {
			return Response.ok("The specified network or application are invalid or configuration was missing from the application settings.").status(Response.Status.BAD_REQUEST).build();
		}
	}
	
	
	@GET
	@Path("/verify/{uid}/{verifier}")
	@Produces("application/xml")
	public VerificationResponse verify(@PathParam("uid") String uid,@PathParam("verifier") String verifier,
					@QueryParam("daliKey") @DefaultValue("") String appKey) throws DaliCoreException {
		VerificationResponse response = new VerificationResponse();
		response.verified = dcsBean.verifyVerifier(uid, verifier);
		return response;
	}
	

	@XmlRootElement
	public static final class VerificationResponse {
		private boolean verified;
	}
	

	//@GET
	//@Path("/accounts/{uid}")
	public List<OnlineAccount> getAccounts(@PathParam("uid") String uid,
					@QueryParam("daliKey") @DefaultValue("") String appKey) throws DaliCoreException {
		User user = userBean.getByUid(appKey, uid);
		if (user != null) {
			return dcsBean.findOnlineAccountsByUser(appKey, user.getId());
		}
		return new LinkedList<OnlineAccount>();
	}

	//@GET
	//@Path("/stream/{uid}")
	public List<Stream> getStream(@PathParam("uid") String uid,
					@QueryParam("daliKey") @DefaultValue("") String appKey) throws DaliCoreException {
		Map<String, String> initParameters = new HashMap<String, String>();
		for (Enumeration<String> initParameterNames = servletContext.getInitParameterNames(); initParameterNames.hasMoreElements();) {
			String initParameterName = initParameterNames.nextElement();
			initParameters.put(initParameterName, servletContext.getInitParameter(initParameterName));
		}
		List<Stream> stream = new LinkedList<Stream>();
		stream.addAll(dcsBean.getStream(appKey, uid, initParameters));
		return stream;
	}

	//@GET
	//@Path("/friends/{uid}")
	public List<Friend> getFriends(@PathParam("uid") String uid,
					@QueryParam("daliKey") @DefaultValue("") String appKey) throws DaliCoreException {
		Map<String, String> initParameters = new HashMap<String, String>();
		for (Enumeration<String> initParameterNames = servletContext.getInitParameterNames(); initParameterNames.hasMoreElements();) {
			String initParameterName = initParameterNames.nextElement();
			initParameters.put(initParameterName, servletContext.getInitParameter(initParameterName));
		}
		List<Friend> friends = new LinkedList<Friend>();
		friends.addAll(dcsBean.getFriends(appKey, uid, initParameters));
		return friends;
	}
}
